function events = FRevents(subjectno, session, dataroot)
%
% FREVENTS - creates a free recall events structure
%
% Only works on one subjects session. Uses subject's session using information
% from par file, word pool file, and free recall logfile.
%
% FUNCTION: events = FRevents(subjectno, session, dataroot)
%
% INPUT ARGS: 
%   subjectno:    subject number; this is the name of the subject folder
%   session:        session number           
%   dataroot: directory of where the data is located
%
% OUTPUT ARGS: 
%   events structure with the following fields
%         1) subject = subject number
%         2) session = session number
%         3) type    = event type (ex. 'TRIAL', 'REC_START')
%         4) item    = event item (ex. 'BOOK', 'DOG' if type is 'WORD)
%         5) itemno = the number of word in the word
%            pool, -1 if it is not, -999 if it is anything else
%         6) list    = list number 
%         7) recalled =  1 for recalled, 0 for not recalled words
%         8) mstime = time each event occurs including when the words are recalled            
%         9) msoffset = inaccuracy of runtime
%        10) rectime = time word was recalled
%        11) intrusion = XLI(-1), word recalled (0), VV-vocalizaion (-999) any
%            other number (refers to the number of lists PLI is
%                             from. e.g. 3. PLI comes from 3 lists back)
%       
% SAMPLE INPUTS:
%  
% clear all;
% subjectno = 'FR008'; 
% session = 1;
% dataroot = '/Users/vitaly/Desktop/data/';


% load in LOGFILE & WORDPOOL
FRlogfile =  fullfile(dataroot, subjectno, ['session_' num2str(session-1) ],'session.log');
[time1, maxoffset1, type1, trial1, word1] = textread(FRlogfile , '%s%s%s%s%s' , 'emptyvalue',  NaN );

wordpool_file = fullfile(dataroot, subjectno, ['session_' num2str(session-1) ],'wordpool.txt');
werdpool = textread(wordpool_file, '%s'); % read in the word pool

% number of trials and list length of words in the experiment
trials = length(find(strcmp(type1, 'TRIAL')));
listlength = length(find(strcmp(type1, 'WORD')))/trials;

% variables to be used for storing info from PARFILE
rec_time = zeros(trials, listlength);% time word was recalled 
posrec = zeros(trials, listlength); % position of word that was recalled ;
recwords = [];
rec_time(rec_time==0) = NaN; 
posrec (posrec==0) = NaN;
rec_per_listE = []; % number of recalled words per list

% load in PARFILE 
for lis=1:trials
     var =fullfile(dataroot, subjectno, ['session_' num2str(session-1) ],[num2str(lis-1) '.par' ]);
     % var = sprintf ( '%s%s/session_%d/%d.par', dataroot, subjectno, session-1, (lis-1));
     [times, posrecalled, wordrecalled] = textread (var , '%n%n%s' , 'emptyvalue', NaN);          
     recwords = [recwords wordrecalled']; % recalled words
     % store times and positions of recalled words  
     for  x= 1:length(times)
       rec_time(lis, x) = times(x); % times of recalled words per list basis
       posrec (lis, x) = posrecalled(x); % position of recalled words per list basis 
     end   
     rec_per_listE = [rec_per_listE length(times)]; % number of recalled words per list
end

% variables to be used in loading struct in [1 by length(events)] format
type      = [];          % either 'TRIAL' (Start of list) , 'WORD' , SESS_START,  REC_START, ORIENT 
word      = [];          % the actual word itself
time      = [];          % time of each event
maxoffset = [];          % how accurate is our time 
rec_pos = [];            % position of the word that was recalled from par file
par=1;

% cleans up the spaces from textread & annotes events for words that were
% recalled using information from par files
for c=1:length(type1) 
   if ~isempty(type1{c})
      type = [type type1(c)];
      word = [word word1(c)];
      time = [time str2num(time1{c})];    
      maxoffset = [maxoffset str2num(maxoffset1{c})];
     
      % adds events for recalled words from the par files
      if strcmp(type1(c), 'REC_START')      
          for k = 1:rec_per_listE(par)
             type1{c} = 'REC_WORD';         
             type = [type type1(c)];
             word = [word word1(c)];
             time = [time str2num(time1{c}) + rec_time(par,k)];  % adds the time the word was recalled to REC_START time
             maxoffset = [maxoffset -999];
          end 
        par=par+1;
     end   
   end    
end

% create events structure
event = struct('subject',[], 'session', [], 'list', [], 'serialpos',[], 'type', [],...
             'item',[],'itemno',[],'recalled',[],'mstime', [], 'msoffset',...
              [], 'rectime' ,[], 'intrusion', []);
events = repmat(event,1,length(type));

presented_list = zeros(trials, listlength); % words that were presented in the list
recword_counter = 1;

% FILL EVENTS STRUCTURE
lis = 0;
for e=1:length(type)
    
  events(e).type = type{e};
  events(e).msoffset = -999 ;
  events(e).mstime = time(e); 
  events(e).list =-999;
  events(e).serialpos = -999;
  events(e).subject  = subjectno;
  events(e).session = session;
  events(e).itemno = -999;
  events(e).recalled = -999;
  events(e).item = 'X'; 
  events(e).rectime = -999; % time word was recalled after recording started 
  events(e).intrusion = -999;
  
   switch type{e}
       
      case{'TRIAL'}
          events(e).msoffset = maxoffset(e);
          lis = lis + 1; % current list
          werd=1;
          rectime_counter = 1;
                       
      case{'ORIENT'}
         events(e).msoffset = maxoffset(e);
          
      case{'WORD'}          
          events(e).item = word{e}; % word in 
          events(e).list = lis; % list number
          itemno = find (strcmp(word{e}, werdpool)); % finds what position in the word pool the presented word is in
          events(e).itemno = itemno; % item number in the word pool
          events(e).msoffset = maxoffset(e);
          events(e).serialpos = werd;
          presented_list(lis,werd) = itemno;
          werd= werd+1; % next position
          
          
                    
          % recalled or not
          if find(itemno == posrec(lis,:))
          where = find  (  posrec(lis,:)  == itemno);
          events(e).recalled = 1;
          events(e).rectime = rec_time(lis, where(1));
          else 
          events(e).recalled = 0;
          end
                       
       case{'REC_WORD'}   
          events (e).itemno = posrec(lis, rectime_counter); % position of recalled word from the parse file
          events(e).rectime = rec_time(lis, rectime_counter);
          events(e).item = recwords{recword_counter};
                   
          % counters for par file variables
          rectime_counter = rectime_counter + 1; 
          recword_counter = recword_counter + 1; 
          events(e).list = lis; % list number
          
          % **********INTRUSIONS******************************************
          % xli = -1;
          % pli = age of intrusion ( number lists back the word first appeared)
          % recalled word from list = 0;
          % -999 if it is anything else;
        
         % if it is not a vocalization, then proceed to figure out if it is
         % an intrusion and then what kind of intrusionit is
         if ~strcmp (events(e).item, 'VV')
          
                % XLI:   if the recalled word is not on the present list nor has the word ever shown up before 
               if isempty(find(events(e).itemno == presented_list)) 
                    events(e).intrusion = -1;   %it is an xli
               end
                     
                % PLI:  if the recalled word is not on the present list    but it on one of the prior lists    
                if events(e).itemno ~= presented_list (lis,:)           & length (find (events(e).itemno  == presented_list (lis-1:-1:1, :)))  == 1  
                    %find in what list did this occur and in what position did the pli occur
                    [list,pos] =  find ( presented_list ( 1:1:lis-1, :)  == events(e).itemno ); 
                     events(e).intrusion = lis-list;  %age of PLI intrusion                 
               end
             
         
         % if it is a vocalization then fill in the appropriate fields
         else
             events(e).type = 'REC_WORD_VV';
             events(e).intrusion = -999;
             events(e).itemno = -999;
         end
         
         % for the words that were recalled make the intrusion field be 0
         if find(events(e).itemno==presented_list(lis,:))
                events(e).intrusion(events(e).intrusion==-999) = 0;
         end
             
     end
end



