#!/usr/bin/python

from pyepl.locals import *


# other modules
import random
import time
import RandomArray
import os
import sys
import shutil

# Set the current version
VERSION = '0.3.0'
MIN_PYEPL_VERSION = '1.0.0'

def verifyFiles(config):
    """
    Verify that all the files specified in the config are there so
    that there is no random failure in the middle of the experiment.

    This will call sys.exit(1) if any of the files are missing.
    """

    # make the list of files from the config vars
    files = (config.RECOG_instructions,
             config.wp,
             config.practice_instruct,
             config.instruct,
             config.defaultFont)

    for f in files:
        if not os.path.exists(f):
            print "\nERROR:\nPath/File does not exist: %s\n\nPlease verify the config.\n" % f
            sys.exit(1)
    
def makeStimForms(exp, trialList):
    """ 
    Generate and compile LaTeX code containing a table with each
    trial's words on a row, with checkboxes and fields for keeping
    track of a hospital stim-session.  Currently this can cope with
    trials of list length up to 5.
    """
    print "Making latex form..."
    formName = "stimForm"
    stimForm = exp.session.createFile(formName+'.tex')
    # how many words per trial:
    wpt = len(trialList[0])
    fontsize = 12
    if wpt>3:
	leftval = -2
	if wpt>4:
	    fontsize = 10
    else:
	leftval = -1
    preamble = []
    # right now I don't know how to rotate a longtable, so we're
    # coping with the longer list values (just barely) by increasing
    # the negative margin on the left and shrinking the fontsize.
    preamble.append("\\documentclass[%dpt]{article}\n" % fontsize)
    preamble.append("\\usepackage{amssymb}\n")
    preamble.append("\\usepackage{longtable}\n")
    preamble.append("\\usepackage{graphicx}\n")
    preamble.append("\\setlength{\\oddsidemargin}{%dcm}\n" % leftval)
    preamble.append("\\setlength{\\evensidemargin}{%dcm}\n" % leftval)
    # technically, we're no longer in the preamble here...
    preamble.append("\\begin{document}\n")
    preamble.append("\\begin{enumerate}\n")
    preamble.append("\\item Patient Name OR Subject Code: \_\_\_\_\_\_\_\_\_\_\_\_\_\_\_\_\\\\\n")
    preamble.append("\\item Date: \_\_\_\_\_\_/\_\_\_\_\_\_/\_\_\_\_\_\_\\\\\n")
    preamble.append("\\end{enumerate}\n")

    # declare 2 columns (word + checkbox) for each word in a trial
    table_format = ''
    for i in range(wpt):
	table_format += 'cc'
    preamble.append("\\begin{longtable}{c%sccccc}\n" % table_format)

    # make column headers:
    headerline = 'trial &'
    for i in range(wpt):
	headerline += 'word$_%d$ & & ' % (i+1)
    preamble.append("%sstim & mAmps & contact$_1$ & contact$_2$ & AD\\\\\n" % headerline)
    preamble.append("\\hline \\hline \\endhead\n")
    stimForm.writelines(preamble)

    # now code the table rows
    for i, trial in enumerate(trialList):
	line = '\\rule[0mm]{0mm}{10mm}%d & ' % (i+1)
	for word in trial:
	    line += word['name']+' & $\\square$ & '
	stimForm.writelines('%s$\\square$ & \\_\\_\\_\\_ & \\_\\_\\_\\_ & \\_\\_\\_\\_ & $\\square$\\\\ \\hline\n' % line)

    # finish up
    postamble = []
    postamble.append("\\end{longtable}\n")
    postamble.append("\\end{document}\n")
    stimForm.writelines(postamble)
    stimForm.close()

    # compile the LaTeX
    print "Compiling latex form..."
    os.chdir(exp.session.fullPath())
    command = 'pdflatex %s' % formName+'.tex'
    os.system(command)
    os.system(command) # need to run twice to get formatting right (!?)
    os.chdir('../../../')

def prepare(exp, config):
    """
    Prepare the trials...
    """

    # verify that we have all the files
    verifyFiles(config)

    # get the state
    state = exp.restoreState()
    
    # load the word pool
    wp = Pool(config.wp)

    # copy the word pool to the sessions dir
    shutil.copy(config.wp,exp.session.fullPath())
    
    sessionList = []
    sessionTargets = []
    sessionLures = []
    for sessionNum in xrange(config.numSessions):
        # Set the session so that we know the correct directory
        exp.setSession(sessionNum)
        
        # Get the session-specific config
        sessionconfig = config.sequence(sessionNum)
        
        # Shuffle the word pool
        wp.shuffle()
        sliceStart = 0
        trialList = []
        for trialNum in xrange(sessionconfig.numTrials):
            # Get the trial specific config
            trialconfig = sessionconfig.sequence(trialNum)
            
            # slice the list
            currentList = wp[sliceStart:sliceStart+trialconfig.listLen]
            trialList.append(currentList)

            # write the list out to file
            listFile = exp.session.createFile("%d.lst" % trialNum)
            for eachWord in currentList:
                listFile.write("%s\n" %
                               getattr(eachWord,sessionconfig.presentationAttribute))
            listFile.close()
            
            # update the slice start
            sliceStart += trialconfig.listLen

        # append the trials to the session
        sessionList.append(trialList)

	try:
	    if config.makeStimForms:
		makeStimForms(exp, trialList)
	except AttributeError:
	    pass

        # see if setting up recognition task
        if sessionconfig.doRecognition:
            # get the targets and lures
            #words = RandomArray.permutation(sliceStart-1)[:sessionconfig.numTargets]
            #print words
            #targets = wp[words.astype(RandomArray.IntType).tolist()]
            targets = wp[:sliceStart]
            targets.shuffle()
            targets = targets[:sessionconfig.RECOG_numTargets]
            lures = wp[sliceStart:(sliceStart+sessionconfig.RECOG_numLures)]
            sessionTargets.append(targets)
            sessionLures.append(lures)

            # write out targes and lures to file
            targetFile = exp.session.createFile("RECOGtargets.txt")
            for eachWord in targets:
                targetFile.write("%s\n" %
                               getattr(eachWord,sessionconfig.presentationAttribute))
            targetFile.close()
            
            lureFile = exp.session.createFile("RECOGlures.txt")
            for eachWord in lures:
                lureFile.write("%s\n" %
                               getattr(eachWord,sessionconfig.presentationAttribute))
            lureFile.close()

    # save the prepared data
    exp.saveState(state,
                  trialNum=0,
                  sessionList=sessionList,
                  sessionTargets=sessionTargets,
                  sessionLures=sessionLures,
                  sessionNum=0)


def run(exp, config):
    """
    Run a session of free recall.
    """

    # set priority
    if config.doRealtime:
        setRealtime(config.rtPeriod,config.rtComputation,config.rtConstraint)
    
    # verify that we have all the files
    verifyFiles(config)
    
    # get the state
    state = exp.restoreState()
    
    # set up the session
    # have we run all the sessions
    if state.sessionNum >= len(state.sessionList):
        print "No more sessions!"
        return

    # set the session number
    exp.setSession(state.sessionNum)
    
    # get session specific config
    sessionconfig = config.sequence(state.sessionNum)

    # create tracks...
    video = VideoTrack("video")
    audio = AudioTrack("audio")
    keyboard = KeyTrack("keyboard")
    log = LogTrack("session")
    mathlog = LogTrack("math")

    
    # set the default font
    setDefaultFont(Font(sessionconfig.defaultFont))
    
    # get a presentation clock
    clock = PresentationClock()

    # log start
    timestamp = clock.get()
    log.logMessage('SESS_START\t%d' % (state.sessionNum + 1), timestamp)

    # do instructions on first trial of each session
    if state.trialNum == 0:
        # do miketest
        soundgood = micTest(2000,1.0)
        if not soundgood:
            # quit
            return
	
        # do the math distract practice if wanted
        if sessionconfig.doMathDistract or sessionconfig.continuousDistract:
            # show practice instructions
	    video.clear("black")
	    instruct(open(sessionconfig.practice_instruct,'r').read())
            
	    mathDistract(clk = clock,
                         mathlog = mathlog,
                         numVars = sessionconfig.MATH_numVars,
                         maxProbs = sessionconfig.MATH_maxProbs,
                         plusAndMinus = sessionconfig.MATH_plusAndMinus,
                         minDuration = sessionconfig.MATH_minDuration_Practice,
                         textSize = sessionconfig.MATH_textSize)

	# Show main instructions
	video.clear("black")
        instruct(open(sessionconfig.instruct,'r').read())


    # present each trial in the session
    while state.trialNum < len(state.sessionList[state.sessionNum]):
        # load trial specific config
        trialconfig = sessionconfig.sequence(state.trialNum)

	# create the beeps
	startBeep = Beep(trialconfig.startBeepFreq,
			 trialconfig.startBeepDur,
			 trialconfig.startBeepRiseFall)
	stopBeep = Beep(trialconfig.stopBeepFreq,
			trialconfig.stopBeepDur,
			trialconfig.stopBeepRiseFall)

        # Clear to start
        video.clear("black")

        # show the current trial and wait for keypress
        timestamp = waitForAnyKey(clock,Text("Press any key for Trial #%d" % (state.trialNum + 1)))
        log.logMessage('TRIAL\t%d' % (state.trialNum + 1),timestamp)

        # display the "cross-hairs"
        timestamp = flashStimulus(Text(trialconfig.orientText, size = trialconfig.wordHeight),
                      clk=clock,
                      duration=trialconfig.wordDuration)
        clock.delay(trialconfig.PauseBeforeWords, trialconfig.JitterBeforeWords)

        # log the flash
        log.logMessage('ORIENT', timestamp)

        # start them words
        for n, eachWord in enumerate(state.sessionList[state.sessionNum][state.trialNum]):
            # get word specific config
            wordconfig = trialconfig.sequence(n)

            # create the stimtext if needed
            if wordconfig.presentationType == 'text':
                eachWord.text = Text(getattr(eachWord,wordconfig.presentationAttribute),
                                     size = wordconfig.wordHeight)
            
            # present the word
            timestamp = getattr(eachWord,
                                wordconfig.presentationType).present(clk = clock,
                                                                     duration = wordconfig.wordDuration)
            clock.delay(wordconfig.ISI,wordconfig.Jitter)

            # log the word
            log.logMessage('WORD\t%s\t%s\t%d' % (wordconfig.presentationType,eachWord.name,n), timestamp)
            
            # Add optional distractor after each word
            if wordconfig.continuousDistract:
                mathDistract(clk = clock,
                             mathlog = mathlog,
                             numVars = trialconfig.MATH_numVars,
                             maxProbs = trialconfig.MATH_maxProbs,
                             plusAndMinus = trialconfig.MATH_plusAndMinus,
                             minDuration = trialconfig.MATH_minDuration,
                             textSize = trialconfig.MATH_textSize)
                
            
        # do the math distract
        if trialconfig.doMathDistract and not trialconfig.continuousDistract:
            mathDistract(clk = clock,
                         mathlog = mathlog,
                         numVars = trialconfig.MATH_numVars,
                         maxProbs = trialconfig.MATH_maxProbs,
                         plusAndMinus = trialconfig.MATH_plusAndMinus,
                         minDuration = trialconfig.MATH_minDuration,
                         textSize = trialconfig.MATH_textSize)

	# Pause before recall
	clock.delay(trialconfig.PauseBeforeRecall, trialconfig.JitterBeforeRecall)
        
	# show the recall start indicator
	startText = video.showCentered(Text(trialconfig.recallStartText, size = trialconfig.wordHeight))
	video.updateScreen(clock)

        # record recall
	startBeep.present(clock)
#         audio.beep(trialconfig.startBeepFreq,
#                    trialconfig.startBeepDur,
#                    trialconfig.startBeepRiseFall,
#                    t=clock)
	
	# hide rec start text
	video.unshow(startText)
	video.updateScreen(clock)

	# Record responses
        (rec,timestamp) = audio.record(trialconfig.recallDuration,str(state.trialNum),t=clock)
	stopBeep.present(clock)
#         audio.beep(trialconfig.stopBeepFreq,
#                    trialconfig.stopBeepDur,
#                    trialconfig.stopBeepRiseFall,
#                    t=clock)

        # log the recstart
        log.logMessage('REC_START' % (),timestamp)
        
        # update the trialnum state
        state.trialNum += 1

        # save the state after each trial
        exp.saveState(state)

    # do the recognition task if wanted
    if sessionconfig.doRecognition:
        # show instructions
        video.clear("black")
        waitForAnyKey(clock,Text("Thank you!\nPlease let the experimenter know\nthat you have finished this\npart of the experiment."))
        instruct(open(sessionconfig.RECOG_instructions,'r').read())

        # get the current targets and lures
        targets = state.sessionTargets[state.sessionNum]
        lures = state.sessionLures[state.sessionNum]
        
        # see if must make the texts
        if sessionconfig.presentationType == 'text':
            targets.makeTexts(sessionconfig.presentationAttribute,
                              size = sessionconfig.wordHeight)
            lures.makeTexts(sessionconfig.presentationAttribute,
                              size = sessionconfig.wordHeight)
      
        # do the recognition
        recognition(targets,
                    lures,
                    sessionconfig.presentationType,
                    clk = clock,
                    log = None,
                    ISI = sessionconfig.RECOG_ISI,
                    ISI_jitter = sessionconfig.RECOG_ISI_jitter,
                    minDuration = sessionconfig.RECOG_minDuration,
                    duration = sessionconfig.RECOG_maxDuration,
                    targetKey = sessionconfig.RECOG_targetKey,
                    lureKey = sessionconfig.RECOG_lureKey)
        
    # save the state when the session is finished
    exp.saveState(state, trialNum = 0, sessionNum = state.sessionNum + 1)

    # Done
    timestamp = waitForAnyKey(clock,Text("Thank you!\nYou have completed the session."))
    log.logMessage('SESS_END',timestamp)

    # Catch up
    clock.wait()


# only do this if the experiment is run as a stand-alone program (not imported as a library)
if __name__ == "__main__":
    # make sure we have the min pyepl version
    checkVersion(MIN_PYEPL_VERSION)
    
    # start PyEPL, parse command line options, and do subject housekeeping
    exp = Experiment()
    exp.parseArgs()
    exp.setup()
    
    # allow users to break out of the experiment with escape-F1 (the default key combo)
    exp.setBreak()
    
    # get subj. config
    config = exp.getConfig()

    # if there was no saved state, run the prepare function
    if not exp.restoreState():
        prepare(exp, config)

    # now run the subject
    run(exp, config)
    
